/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#ifndef MTK_DMX_H
#define MTK_DMX_H

#include "dmx_if.h"

// The input pin set
/*****************************************************************
1: DMX_PINMUX_EXT_S = 0x10,           // Serial
        //PAD_CGPIO15  -----> TSCLK
        //PAD_CGPIO16  ----->TSVAL
        //PAD_CGPIO17  ----->TSSYNC
        //PAD_CGPIO18  ----->TSDATA0
        
2: DMX_PINMUX_EXT_P = 0x11,           // Parallel
        //PAD_CGPIO15  ----->TSCLK
        //PAD_CGPIO16  ----->TSVAL
        //PAD_CGPIO17  ----->TSSYNC
        //PAD_CGPIO18  ----->TSDATA0
        //PAD_CGPIO19  ----->TSDATA1
        //PAD_CGPIO20  ----->TSDATA2
        //PAD_CGPIO21  ----->TSDATA3
        //PAD_CGPIO22  ----->TSDATA4
        //PAD_CGPIO23  ----->TSDATA5
        //PAD_CGPIO24  ----->TSDATA6
        //PAD_CGPIO25  ----->TSDATA7
        
3: DMX_PINMUX_EXT_1_S = 0x20,         // Serial
        //PAD_CGPIO19  ----->TSCLK (FUNCTION 4)
        //PAD_CGPIO20  ----->TSVALID
        //PAD_CGPIO21  ----->TSSYNC
        //PAD_CGPIO22  ----->TSDATA0
        
4: DMX_PINMUX_EXT_2_S = 0x30,         // Serial
        //PAD_CGPIO48  ----->TSCLK
        //PAD_CGPIO46  ----->TSVALID
        //PAD_CGPIO45  ----->TSSYNC
        //PAD_CGPIO34  ----->TSDATA0
        //PAD_CGPIO35  ----->TSDATA1
        
5: DMX_PINMUX_EXT_2_P = 0x31,         // Parallel,
        //PAD_CGPIO48  ----->TSCLK
        //PAD_CGPIO46  ----->TSVALID
        //PAD_CGPIO45  ----->TSSYNC
        //PAD_CGPIO34  ----->TSDATA0
        //PAD_CGPIO35  ----->TSDATA1
        //PAD_CGPIO36  ----->TSDATA2
        //PAD_CGPIO37  ----->TSDATA3
        //PAD_CGPIO38  ----->TSDATA4
        //PAD_CGPIO39  ----->TSDATA5
        //PAD_CGPIO40  ----->TSDATA6
        //PAD_CGPIO41  ----->TSDATA7
        
6: DMX_PINMUX_EXT2_S = 0x40,          // Serial
        //PAD_CGPIO36  ----->TSCLK (FUNCTION 2)
        //PAD_CGPIO37  ----->TSVALID
        //PAD_CGPIO38  ----->TSSYNC
        //PAD_CGPIO39  ----->TSDATA0
        //PAD_CGPIO40  ----->TSDATA1
        
7: DMX_PINMUX_EXT3_S = 0X50,          // Serial
        //PAD_MCM_GPIO3  ----->TSCLK
        //PAD_MCM_GPIO4  ----->TSVALID
        //PAD_MCM_GPIO5  ----->TSSYNC
        //PAD_MCM_GPIO6  ----->TSDATA0
*****************************************************************/

// Framer Mode
//  DMX_FRAMER_SERIAL: 1 bit data
//  DMX_FRAMER_PARALLEL: 8 bits data
//  DMX_FRAMER_TWOBIT: 2 bits data for MT5135

// Edge Setting
//  DMX_EDGE_POSITIVE: positive edge sampling
//  DMX_EDGE_NEGATIVE: negative edge sampling

// Sync Control
//  DMX_SYNC_INTERNAL: sync by data is 0x47
//  DMX_SYNC_EXTERNAL: sync by sync singal

static const DMX_FRONTEND_CONFIG_T _arMtkFeCfgTable[] =
{
#ifdef CC_EXTERNAL_DEMOD
#ifdef CC_ISDB_SUPPORT
    // Frontend input path 0
    { TRUE, DMX_PINSET_EXT1, DMX_FRAMER_SERIAL, DMX_EDGE_NEGATIVE, DMX_SYNC_INTERNAL},    
#else
    // Frontend input path 0
    { TRUE, DMX_PINSET_EXT1, DMX_FRAMER_SERIAL, DMX_EDGE_POSITIVE, DMX_SYNC_INTERNAL},
#endif        
#else
    // Frontend input path 0
    { TRUE, DMX_PINSET_INTERNAL, DMX_FRAMER_SERIAL, DMX_EDGE_POSITIVE, DMX_SYNC_INTERNAL},
#endif
    // Frontend input path 1
    { FALSE, DMX_PINSET_EXT2_0, DMX_FRAMER_PARALLEL, DMX_EDGE_POSITIVE, DMX_SYNC_INTERNAL},
    // PVR path
    { FALSE, DMX_PINSET_INTERNAL, DMX_FRAMER_SERIAL, DMX_EDGE_POSITIVE, DMX_SYNC_INTERNAL},
    // Frontend input path 2
    { FALSE, DMX_PINSET_EXT3, DMX_FRAMER_TWOBIT, DMX_EDGE_POSITIVE, DMX_SYNC_INTERNAL},
};

#define DMX_FRONTEND_CONFIG_TABLE     (&_arMtkFeCfgTable)
#define DMX_FRONTEND_CONFIG_SIZE      (sizeof(_arMtkFeCfgTable)/sizeof(DMX_FRONTEND_CONFIG_T))

static const DMX_SCRAMBLE_CONFIG_T _rMtkDmxScrambleConfig =
{
    DMX_SCRAMBLE_PESSTART_ONLY,   // DMX_SCRAMBLE_TSFLAG_ONLY
    0,                            // If 
};

#define DMX_SCRAMBLE_CONFIG_TABLE     (&_rMtkDmxScrambleConfig)

#endif /* MTK_DMX_H */
