#ifndef SALVIA_COMMON_PCB_DRV_CUST_H
#define SALVIA_COMMON_PCB_DRV_CUST_H 1

//----------------------------------------------------------------------------
// Initial dram settings
//----------------------------------------------------------------------------
#define DEFAULT_DRAM_TYPE                   DDR_III_x1
#define DEFAULT_DDR_CLOCK                   (1404000000)//(1296000000)

#define DDR3_CMD_DRIVING			0x88
#define DDR3_CS_DRIVING 			0x88


#ifdef CC_FORCE_DRAM256M
#define TOTAL_MEM_SIZE                      (256 * 1024 * 1024)
#else
#define TOTAL_MEM_SIZE                      (512 * 1024 * 1024)
#endif

#ifndef FBM_MEM_CFG_SIZE
#ifdef CC_FORCE_DRAM256M
#define FBM_MEM_CFG_SIZE                    (FBM_MEM_CFG_MT5880_DDR3x1)
#else
#define FBM_MEM_CFG_SIZE                    (FBM_MEM_CFG_MT5880_DDR3x2)
#endif
#endif /* FBM_MEM_CFG_SIZE */


#define DRVCUST_EEPROM_DATA_MEM_OFFSET ((UINT32)0x2F00000)  

#ifdef CC_TRUSTZONE_SUPPORT
#define TRUSTZONE_CODE_BASE                 (0x5000)
#if defined(CC_SVP_SUPPORT)
#define TRUSTZONE_MEM_SIZE                  (16 * 1024 * 1024)
#else
#define TRUSTZONE_MEM_SIZE                  (2 * 1024 * 1024)
#endif
#else
#define TRUSTZONE_MEM_SIZE                  (0)
#endif

//----------------------------------------------------------------------------
// Default Panel setting
//----------------------------------------------------------------------------
#ifndef DEFAULT_PANEL_SELECT
#define DEFAULT_PANEL_SELECT                (PANEL_DEFAULT)
#endif
#define BACKLIGHT_GPIO                      (GPIO(65))
#define BACKLIGHT_ON_VAL                    (1)
#define PANELPOWER_GPIO                     (GPIO(64))
#define PANELPOWER_ON_VAL                   (1)

// disable spread spectrum for vga output
#define SPREAD_SPECTRUM_PERMILLAGE          (0)

// pwm backlight adjustment setting
#define PANEL_BACKLIGHT_PWM_PORT            (2)

#define PDWNC_POLARITY                      (0)

#define SYS_RESET_TIMER             (IS_XTAL_27MHZ()? 0x2932E0 :0x249F00) //100ms

#ifndef DRV_LOGO_ENABLE
#define DRV_LOGO_ENABLE                         (1)
#endif
#define DRV_LOGO_H_FILE                         "Sony/sony_DrvLogo.h"
#define LOADER_MUSIC_H_FILE                     "Sony/sony_DrvLogoMusic.h"
#define DRV_LOGO_EARLY_RETURN_COUNTER          (0x6)

//----------------------------------------------------------------------------
// Audio Customization
//----------------------------------------------------------------------------
//audio
//-----------------------------------------------------------------------------
// AUDIO_Amp_Gpio_Num:         AMP mute
// AUDIO_Amp1_Gpio_Num:       Assist AMP mute (PAA/PAD)
// AUDIO_CUST_HP_GPIO_CTRL:    HP mute (HP/LINEOUT share HW port)
// AUDIO_CUST_SPEC_GPIO_CTRL:  SCART mute (EU)
// AUDIO_CUST_SPEC2_GPIO_CTRL: Hotel Bath SPK mute (PAD) 
//-----------------------------------------------------------------------------
#define AUDIO_INCLUDE_FILE                  "Sony/salvia_aud.h"

#define AUDIO_Amp_Gpio_Num                  80//PIN_TCON0
#define AUDIO_AMP_GPIO_POLARITY             AUD_AMP_GPIO_HIGH_ENALBE //High:Unmute Low:Mute

#define AUDIO_CUST_HP_GPIO_CTRL             86//PIN_TCON6 
#define AUDIO_CUST_HP_POLARITY              AUD_GPIO_HIGH_ENALBE    //High:Unmute Low:Mute

#define AUDIO_HP_PLUG_IN_GPIO               GPIO(92)//PIN_TCON12
#define AUDIO_HP_PLUG_IN_POLARITY           AUD_GPIO_HIGH_ENALBE   //Hihg: Plug in

#define AUDIO_DIGITAL_AMP_RESET_GPIO            85//PIN_TCON5 
#define AUDIO_DIGITAL_AMP_RESET_GPIO_POLARITY   AUD_GPIO_HIGH_ENALBE   //AMP RST High: Normal Low:Mute


#define AUDIO_SPK_TYPE_LARGE                (FALSE)
#define AUDIO_SUBWOOFER_ON                  (TRUE)
#define AUDIO_DOWNMIX_WITH_BALANCE          (FALSE)

#define AUDIO_OUT_PAD_DRV_CUR               (6)

#define AUDIO_INPUT_SW_GPIO_SEL             (&_arSonyInputSwGpio)
#define AUDIO_AOUT_PAD_CFG                  (&_arSonyAoutPadTable)
#define AUDIO_INPUT_MUX_SEL_TABLE           (&_rSonyAudInputMuxTable)
#define AUDIO_ADAC_USAGE                    (&_arSonyAdacUsage)
#define AUDIO_INTER_DAC_USAGE               (&_arSonyInterDacUsage)
#define AUDIO_INTER_DAC_USAGE_ES2           (&_arSonyInterDacUsage)
#define AUDIO_Digital_Data_TABLE            (&_afgSonyAudDigitalDataUsage)
#define AUDIO_INPUT_SRC_VOL                 (&_aai2SonyAudInputSrcVol)
#define AUDIO_AV_INPUT_SRC_VOL              (&_ai2SonyAudAvInputSrcVol) 
#define AUDIO_VOL_TABLE                     (&_au4Sony_VolumeTable) 
#define AUDIO_CHANNEL_VOL_TABLE             (&_au4Sony_VolumeTable)
#define AUDIO_HEADPHONE_VOL_TABLE           (&_au4Sony_VolumeTable)
#define AUDIO_CHL_VOL_EXTRA_GAIN            (&_aai1SonyAudChVolExtraGain)
#define AUDIO_OUT_FMT_TABLE                 (&_arSonyAudOutFmtTable)
#define AUDIO_POWER_ON_MUSIC_VOL            (20)
#define AUDIO_IEC_COPYRIGHT                 (&_au1SonyAudIecCopyright)
#define AUD_CHL_SOUND_MODE_CFG              (&_arSonySoundModeCase)
#define AUDIO_AVC_TARGET_LEV                (&_ai2SonyAudAvcTarget)
#define AUDIO_AVC_GAIN_UP                   (&_ai2SonyAudAvcGainUp)

#ifdef CC_AUD_USE_FLASH_AQ
#ifdef CC_SONY_2K14_FLASH_AQ
#define AUDIO_FLASH_AQ_DATA_DSCRPT          (&_arSonyAudNvmDscrpt)
#define AUDIO_FLASH_AQ_DATA_DSCRPT_SIZE     (u1SonyAudAQDscrptSize)
#else ///def CC_SONY_2K15_FLASH_AQ
#define AUDIO_FLASH_AQ_DATA_DSCRPT          (&_arSonyAudSysNvmDscrpt)
#define AUDIO_FLASH_AQ_DATA_DSCRPT_SIZE     (u1SonyAudSysAQDscrptSize)
#define AUDIO_FLASH_AQ_DATA_DSCRPT1         (&_arSonyAudTuningNvmDscrpt)
#define AUDIO_FLASH_AQ_DATA_DSCRPT_SIZE1    (u1ISonyAudTuningAQDscrptSize)
#endif
#endif
#define FLASH_AQ_BASE_PARTITION             (16)  
#define AUDIO_BI2S_ENABLE                   (FALSE)
#define AUDIO_MTS_FACTORY_MODE              (&_arSonyMtsFactory)
#define AUDIO_A2_FACTORY_MODE               (&_arSonyA2Factory)
#define AUDIO_PAL_FACTORY_MODE              (&_arSonyPalFactory)
#define AUDIO_SPDIF_ENABLE                  (FALSE)
#define AUDIO_DOWNMIX_CHANNEL_POSITION      (DOWNMIX_CH_BEFORE_AVC) 
#define AUDIO_MTK_VS_CLARITY                (0x200000)
#define AUDIO_MTK_VS_WIDTH                  (0x300000)
#define AUDIO_MTK_VS_XTALK                  (0x022000)
#define AUDIO_MTK_VS_OUTPUTGAIN             (0x043000)
#define AUDIO_MTK_VS_BASSGAIN               (0x060000)
#define AUDIO_LINE_OUT_FIX_VOL              (88)

#define AUDIO_HEAAC_MULTICHL_DDCO           (FALSE)

#define FLAG_EXT_STATUS     (1)

#define DEFAULT_DEMOD_RESET_GPIO            (GPIO(58))
#define DEFAULT_DEMOD_I2C_BUS_ID  1
#define DEFAULT_TUNER_I2C_BUS_ID 1


//----------------------------------------------------------------------------
// EEPROM customization
//----------------------------------------------------------------------------
#define SYSTEM_EEPROM_ADDRESS             (0xA4)
#ifdef REPLACE_EEPROM_WITH_FLASH
#define SYSTEM_EEPROM_SIZE                  (16*1024)
#else
#define SYSTEM_EEPROM_SIZE                  0x4000//(4096)
#endif
#define SYSTEM_EEPROM_WP_GPIO               (68)
#define SYSTEM_EEPROM_WP_ENABLE_POLARITY    (0)
#define SYSTEM_EEPROM_BUS                   (1)

#define EDID_INCLUDE_FILE                "Sony/salvia_edid.h"

//----------------------------------------------------------------------------
// USB Customization
//----------------------------------------------------------------------------
// 1. Loader/Nuclues system, USB port using define (1)=used (0)=not used
#define USB0_PORT_USING_LOADER              (1)
#define USB1_PORT_USING_LOADER              (0)
#define USB2_PORT_USING_LOADER              (0)
#define USB3_PORT_USING_LOADER              (0)

// 2. Linux/Android System USB port using define (1)=used (0)=not used
#define USB0_PORT_USING                     (1)
#define USB1_PORT_USING                     (1)
#define USB2_PORT_USING                     (0)
#define USB3_PORT_USING                     (0)




// 3. USB power switch Vbus GPIO control, if vbus not control by GPIO, mark off it.
#ifndef CC_IS_SIMPLE_IPTV
#define USB0_VBUS_GPIO   (200) //(63)
#define USB0_VBUS_GPIO_POLARITY             (1)
#endif
// 4. USB power switch overcurrent GPIO control, if have not ic to control by GPIO, mark off it.
#ifndef CC_IS_SIMPLE_IPTV
#define USB0_OC_GPIO                        (202) //GPIO(73)
#define USB0_OC_GPIO_POLARITY               (0)
#endif

// 5. Detect USB upgrade time, default is 0, but some device maybe can't detect when upgrade.
// If possible, set it to (300)ms when in customer's module.
#define USB_LOADER_WAIT_TIME                (0)

#ifdef CC_IS_SIMPLE_IPTV
#define USB_HUB_RESET_GPIO (GPIO(82))//PIN_TCON2
#define USB_HUB_RESET_GPIO_POLARITY (0)//L:RESET H:NORMAL
#endif

//----------------------------------------------------------------------------
// WIFI Customization
//----------------------------------------------------------------------------
#define CONFIG_WIFI_PORT_DISABLE 1
#define CONFIG_WIFI_PORT_NUMBER    1//wifi is in usb port 1
#ifdef WIFI_WOWLAN_SUPPORT
#define WAKEUP_ON_WLAN_GPIO OPCTRL(2)
#define WAKEUP_ON_WLAN_POL (1) //high:wakeup
#define WIFI_MODULE_RESET_GPIO    GPIO(87)
#define WIFI_MODULE_RESET_GPIO_POL (1)//High is reset
#define WIFI_MODULE_RESET_DURATION  (2000)//ms
#endif
//----------------------------------------------------------------------------
// Button Customization
//----------------------------------------------------------------------------
#define FLAG_BUTTON_SUPPORT                 (1)
#define BUTTON_INCLUDE_FILE                 "Sony/Salvia_button.h"

#ifndef T8032UP_INCLUDE_FILE
#define T8032UP_INCLUDE_FILE                "Sony/sony_t8032fw.h"
#define T8032UP_OFFSET                      au1T8032FW
#define T8032UP_SIZE                        24576//16384
#endif

//----------------------------------------------------------------------------
// MTK IR only setting
//----------------------------------------------------------------------------
#define IR_INCLUDE_FILE                     "Sony/sirc_ir.h"

#ifndef NPTV_SOURCE_TABLE
#ifdef NEW_BOARD
#define NPTV_SOURCE_TABLE       "Sony/salvia_source_table.c"
#else
#define NPTV_SOURCE_TABLE                   "mtk/source_table.c"
#endif
#endif
//----------------------------------------------------------------------------
// default PIN Setup
//----------------------------------------------------------------------------
#define PINMUX_INCLUDE_FILE                 "Sony/sony_salvia_pin_setup.h"
#define GPIO_INCLUDE_FILE                       "Sony/Salvia_gpio.h"

//----------------------------------------------------------------------------
// default HDMI GPIO control
//----------------------------------------------------------------------------
#define HDMI_GPIO_CONF "Sony/salvia_hdmi_conf.c"

/*
 * default VGA table
*/
#ifndef VGA_INCLUDE_FILE
#define VGA_INCLUDE_FILE                    "Sony/Sony_vga.h"
#endif

//----------------------------------------------------------------------------
// Tcon Customization
//----------------------------------------------------------------------------
// support LVDS and MLVDS both
#define PANEL_DISPLAY_TYPE_ALLOWED_MASK ((1<<PANEL_DISPLAY_TYPE_LVDS) | (1 << PANEL_DISPLAY_TYPE_MLVDS))

#ifndef TCON_CUST_FILE
#define TCON_CUST_FILE                      "mtk/Viper_tcon_cust.c"
#endif
//#define SUPPORT_LOCK_FROM_SRVAD 

//----------------------------------------------------------------------------
// DMX configuration
//----------------------------------------------------------------------------
#define DMX_INCLUDE_FILE                "Sony/salvia_dmx.h"

//----------------------------------------------------------------------------
// SIF configuration
//----------------------------------------------------------------------------
#define SIF_INCLUDE_FILE               "mtk/mtk_sif_cfg.h"

//----------------------------------------------------------------------------
// Loader configuration
//----------------------------------------------------------------------------
#ifdef CC_USE_256MB_NAND
#define MAX_UPGRADE_IMAGE_FILE_SIZE         (256*1024*1024)     /* 256 MB */
#else
#define MAX_UPGRADE_IMAGE_FILE_SIZE         (512*1024*1024)     /* 512 MB */
#endif

#define LOADER_UPGRADE_ONLY_ACON 0

#if defined(LINUX_TURNKEY_SOLUTION) || defined(__KERNEL__)
//----------------------------------------------------------------------------
// Flash Partition configuration
//----------------------------------------------------------------------------
#define NAND_INCLUDE_FILE                   "Sony/sony_flash_layout.h"

#endif //defined(LINUX_TURNKEY_SOLUTION) || defined(__KERNEL__)

/*
 * dynamic backlight parameter
*/

/*
 * OD setting
*/

#define SUPPORT_OD                          (0)

#ifndef CUSTOM_VIDEO_DRV_BODY_FILE
#define CUSTOM_VIDEO_DRV_BODY_FILE              "Sony/video_drv/Salvia_video_drv.c"
#endif

#ifndef CUSTOM_BACKLIGHT_DRV_BODY_FILE
#define CUSTOM_BACKLIGHT_DRV_BODY_FILE          "Sony/video_drv/Salvia_backlight_drv.c"
#endif

#ifndef COLOR_PARAM_INCLUDE_FILE
#define COLOR_PARAM_INCLUDE_FILE                "Sony/PANEL_DEFAULT/color_param.h"
#endif

//----------------------------------------------------------------------------
// D-interface Customization
//----------------------------------------------------------------------------
#ifndef DRV_CUSTOM_CUST_FILE
#define DRV_CUSTOM_CUST_FILE                    "Sony/Soem_linux_d_custom.c"
#endif

#ifndef DRV_CUSTOM_CUST_SPEC_SET
#define DRV_CUSTOM_CUST_SPEC_SET                d_Soem_cust_spec_set
#endif

#ifndef DRV_CUSTOM_CUST_SPEC_GET
#define DRV_CUSTOM_CUST_SPEC_GET                d_Soem_cust_spec_get
#endif

#define PDWNC_EXT_IMPL                          "Sony/Soem_pdwnc.h"

//----------------------------------------------------------------------------
// Flash Panel
//----------------------------------------------------------------------------
#ifdef CC_PANEL_FROM_FLASH
#ifndef LOAD_PANEL_FROM_FLASH_CUSTOM
#define LOAD_PANEL_FROM_FLASH_CUSTOM            CustomPanelLoadTableFromFlash
#define DUMP_PANEL_LIST_FROM_FLASH_CUSTOM       CustomDumpPanelListFromFlash
#define CUSTOM_SHOW_PANEL_INFO_BY_ID(u4Idx)     CustomShowPanelInfoByID(u4Idx)

#define PANEL_VERSION_LENGTH                    16
#endif
#endif  /* CC_PANEL_FROM_FLASH */

#endif
