/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#ifndef X_DRAM_H
#define X_DRAM_H

//----------------------------------------------------------------------------
// DQS definitions
//----------------------------------------------------------------------------
#define MAX_DQS_DELAY       	127
#define MAX_DQS_OUT_DELAY       31
#define MAX_DQ_DELAY        	31
#define MAX_DQ_OUT_DELAY        31
#define CALIB_START         	0x100000
#define CALIB_LEN           	0x100

//----------------------------------------------------------------------------
// DRAM USE general purpose register to be the parameter passing at preloader.
// BIM (IO_VIRT + 0x8100) ~ (IO_VIRT + 0x810F)
//----------------------------------------------------------------------------
#define TCM_SRAM_ADDR       	(IO_VIRT + 0x8000)
#define TCM_DRAM_FLAGS    		(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRB6)))
#define TCM_DRAM_SIZE      		(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRB7)))
#define TCM_DRAM_FLAGS1      	(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRDW0)))
#define TCM_DRAM_FLAGS2     	(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRDW1)))
#define TCM_DRAM_FLAGS3     	(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRDW2)))
#define TCM_DRAM_DLYCELL_PERT     	(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRDW3)))
#define TCM_DRAM_REGBASE		(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRDW4)))
#define TCM_DRAM_DDRPHY_REGBASE	(*((volatile UINT32 *)BIMSWAP(TCM_SRAM_ADDR + REG_RW_GPRDW5)))


#define DRAM_CLOCK_MASK     (0x00000FFFU)
#define DRAM_TYPE_MASK      (0x0000F000U)
#define TYPE_SHIFT       	(12)
#define DRAM_DDR_CL_MASK    (0x000F0000U)
#define CL_SHIFT       		(16)

#define CHB_SUPPORT_SHIFT       (20)
#define DRAM_CHANNELB_SUPPORT   (1U << CHB_SUPPORT_SHIFT)
#define CHB_ACTIVE_SHIFT        (21)
#define DRAM_CHANNELB_ACTIVE    (1U << CHB_ACTIVE_SHIFT)
#define CHA_FORCE32_SHIFT       (22)
#define DRAM_CHA_FORCE32        (1U << CHA_FORCE32_SHIFT)
#define CHB_FORCE32_SHIFT       (23)
#define DRAM_CHB_FORCE32        (1U << CHB_FORCE32_SHIFT)
#define QFP_SHIFT               (24)
#define DRAM_DDR_QFP            (1U << QFP_SHIFT)
#define SWAP16BIT_SHIFT         (25)
#define DRAM_SWAP16BIT          (1U << SWAP16BIT_SHIFT)
#define DMSSON_SHIFT            (26)
#define DRAM_DMSSON             (1U << DMSSON_SHIFT)
#define ASYNCON_SHIFT       	(27)
#define DRAM_ASYNCON        	(1U << ASYNCON_SHIFT)
#define CTSMODE_SHIFT       	(28)
#define DRAM_CTSMODE        	(1U << CTSMODE_SHIFT)
#define BUSX8_SHIFT       	(29)
#define DRAM_BUSX8        	(1U << BUSX8_SHIFT)

#define BASE_DDR_CLK        1000000
#define TCMGET_DDR_CLK()    ((TCM_DRAM_FLAGS & DRAM_CLOCK_MASK) * BASE_DDR_CLK)
#define TCMGET_DRAMTYPE()  	((TCM_DRAM_FLAGS & DRAM_TYPE_MASK) >> TYPE_SHIFT)
#define TCMGET_DDR_CL()     ((TCM_DRAM_FLAGS & DRAM_DDR_CL_MASK) >> CL_SHIFT)
#ifdef CC_MTK_PRELOADER
#define TCMSET_CHANNELA_ACTIVE()    (TCM_DRAM_FLAGS &= ~DRAM_CHANNELB_ACTIVE); \
                                    (TCM_DRAM_REGBASE = DRAM_BASE);\
                                    (TCM_DRAM_DDRPHY_REGBASE = DRAM_DDRPHY_BASE)
#define TCMSET_CHANNELB_ACTIVE()    (TCM_DRAM_FLAGS |= DRAM_CHANNELB_ACTIVE); \
                                    (TCM_DRAM_REGBASE = DRAM_CHB_BASE);\
                                    (TCM_DRAM_DDRPHY_REGBASE = DRAM_DDRPHY_CHB_BASE)
#else
#define TCMSET_CHANNELA_ACTIVE()    (TCM_DRAM_FLAGS &= ~DRAM_CHANNELB_ACTIVE)
#define TCMSET_CHANNELB_ACTIVE()    (TCM_DRAM_FLAGS |= DRAM_CHANNELB_ACTIVE)
#endif /* #ifdef CC_MTK_PRELOADER */

#define IS_DRAM_CHANNELB_SUPPORT()    ((TCM_DRAM_FLAGS & DRAM_CHANNELB_SUPPORT) ? 1 : 0)
#define IS_DRAM_CHANNELB_ACTIVE()    ((TCM_DRAM_FLAGS & DRAM_CHANNELB_ACTIVE) ? 1 : 0)
#define TCMSET_CHANNELA_SIZE(size)    (TCM_DRAM_SIZE = (TCM_DRAM_SIZE & 0xffff0000) + (size & 0xffff))   // size in Mbytes.
#define TCMGET_CHANNELA_SIZE()         (TCM_DRAM_SIZE & 0xFFFF) // size in Mbytes. 
#define TCMSET_CHANNELB_SIZE(size)    (TCM_DRAM_SIZE = (TCM_DRAM_SIZE & 0xffff) + (size << 16))
#define TCMGET_CHANNELB_SIZE()         ((TCM_DRAM_SIZE >> 16) & 0xFFFF) // size in Mbytes.

#define TCMSET_DLYCELL_PERT(delaycell_pert)	TCM_DRAM_DLYCELL_PERT = (IS_DRAM_CHANNELB_ACTIVE() ? \
												((TCM_DRAM_DLYCELL_PERT&0x00ff)|((delaycell_pert)<<8)) \
												: ((TCM_DRAM_DLYCELL_PERT&0xff00)|(delaycell_pert)))
#define TCMGET_DLYCELL_PERT				(IS_DRAM_CHANNELB_ACTIVE() ? \
												(TCM_DRAM_DLYCELL_PERT&0xff00)>>8 \
												: (TCM_DRAM_DLYCELL_PERT&0x00ff))


#define IS_FORCE32()        (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                ((TCM_DRAM_FLAGS & DRAM_CHB_FORCE32) ? 1 : 0) : \
                                                ((TCM_DRAM_FLAGS & DRAM_CHA_FORCE32) ? 1 : 0))
#define IS_DDR_QFP()        ((TCM_DRAM_FLAGS & DRAM_DDR_QFP) ? 1 : 0)
#define IS_DDR_SWAP16BIT()        ((TCM_DRAM_FLAGS & DRAM_SWAP16BIT) ? 1 : 0)
#define IS_DDR_DMSSON()        ((TCM_DRAM_FLAGS & DRAM_DMSSON) ? 1 : 0)
#define IS_DDR_ASYNCON()        ((TCM_DRAM_FLAGS & DRAM_ASYNCON) ? 1 : 0)
#define IS_DDR_CTSMODE()        ((TCM_DRAM_FLAGS & DRAM_CTSMODE) ? 1 : 0)
#define IS_DDR_BUSX8()        ((TCM_DRAM_FLAGS & DRAM_BUSX8) ? 1 : 0)

//lint -emacro({717}, ADD_DDR_CLK)  to disable "do {...} while (0)" lint warning
#define ADD_DDR_CLK()                                                       \
            do {                                                            \
                UINT32 u4AddClk;                                            \
                u4AddClk = (TCM_DRAM_FLAGS & DRAM_CLOCK_MASK);              \
                TCM_DRAM_FLAGS &= ~(DRAM_CLOCK_MASK);                       \
                TCM_DRAM_FLAGS |= ((u4AddClk + (IS_XTAL_24MHZ()? 48 : 54)) & DRAM_CLOCK_MASK);     \
            } while (0)

//lint -emacro({717}, SUB_DDR_CLK)  to disable "do {...} while (0)" lint warning
#define SUB_DDR_CLK()                                                       \
            do {                                                            \
                UINT32 u4SubClk;                                            \
                u4SubClk = (TCM_DRAM_FLAGS & DRAM_CLOCK_MASK);              \
                    TCM_DRAM_FLAGS &= ~(DRAM_CLOCK_MASK);                   \
                TCM_DRAM_FLAGS |= ((u4SubClk - (IS_XTAL_24MHZ()? 48 : 54)) & DRAM_CLOCK_MASK);   \
            } while (0)

// DRAM data read/write macros
#define INIT_DRAM_B_BASE            0x80000000
#define INIT_DRAM_B_CHB_BASE    	0x40000000

#define MEM_READ32(offset)              (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                                    IO_READ32(INIT_DRAM_B_CHB_BASE, (offset)) : \
                                                                    IO_READ32(INIT_DRAM_B_BASE, (offset)))
                                                                    
#define MEM_WRITE32(offset, value)      (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                                    IO_WRITE32(INIT_DRAM_B_CHB_BASE, (offset), (value)): \
                                                                    IO_WRITE32(INIT_DRAM_B_BASE, (offset), (value)))                                                                    

#ifdef CC_MTK_PRELOADER

#define DRAM_READ32(offset)            	IO_READ32(TCM_DRAM_REGBASE, (offset))
#if 1
#define DRAM_WRITE32(offset, value)     IO_WRITE32(TCM_DRAM_REGBASE, (offset), (value))
#else
#define DRAM_WRITE32(offset, value)     do{\
											IO_WRITE32(TCM_DRAM_REGBASE, (offset), (value)); \
											Printf("DRAM W: 0x%08X = 0x%08X.\n", TCM_DRAM_REGBASE + offset, value);\
										}while(0)
#endif

#else
// DRAM controller register read/write macros
#define DRAM_READ32(offset)           (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                                   IO_READ32(DRAM_CHB_BASE, (offset)) : \
                                                                   IO_READ32(DRAM_BASE, (offset)))
#define DRAM_WRITE32(offset, value)   (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                                   IO_WRITE32(DRAM_CHB_BASE, (offset), (value)) : \
                                                                   IO_WRITE32(DRAM_BASE, (offset), (value)))
#endif /* #ifdef CC_MTK_PRELOADER */

#ifdef CC_MTK_PRELOADER

#define DRAM_DDRPHY_READ32(offset)            	IO_READ32(TCM_DRAM_DDRPHY_REGBASE, (offset))
#if 1
#define DRAM_DDRPHY_WRITE32(offset, value)     	IO_WRITE32(TCM_DRAM_DDRPHY_REGBASE, (offset), (value))
#else
#define DRAM_DDRPHY_WRITE32(offset, value)     do{\
												   IO_WRITE32(TCM_DRAM_DDRPHY_REGBASE, (offset), (value)); \
												   Printf("DRAM W: 0x%08X = 0x%08X.\n", TCM_DRAM_DDRPHY_REGBASE + offset, value);\
												}while(0)
#endif

#else
// DRAM controller register read/write macros
#define DRAM_DDRPHY_READ32(offset)           (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                                   IO_READ32(DRAM_DDRPHY_CHB_BASE, (offset)) : \
                                                                   IO_READ32(DRAM_DDRPHY_BASE, (offset)))
#define DRAM_DDRPHY_WRITE32(offset, value)     (IS_DRAM_CHANNELB_ACTIVE() ? \
                                                                   IO_WRITE32(DRAM_DDRPHY_CHB_BASE, (offset), (value)) : \
                                                                   IO_WRITE32(DRAM_DDRPHY_BASE, (offset), (value)))
#endif /* #ifdef CC_MTK_PRELOADER */

// DRAM parameters - Params size in DWORD
#define DRAM_PARAMS_TAG_SIZE    (4)
#define DRAM_PARAMS_BANK_SIZE   (48)
#define DRAM_PARAMS_BANK_COUNT  (4)
#define DRAM_PARAMS_SIZE        DRAM_PARAMS_TAG_SIZE + (DRAM_PARAMS_BANK_SIZE * DRAM_PARAMS_BANK_COUNT)

//Write Delay Index
#define DDR3_CHA_BANK_IDX 	             0
#define DDR3_CHB_BANK_IDX 	             1

#define DDR3_DQ_OUTPUT_DELAY_0L_IDX 	 0
#define DDR3_DQ_OUTPUT_DELAY_0H_IDX 	 1
#define DDR3_DQ_OUTPUT_DELAY_1L_IDX 	 2
#define DDR3_DQ_OUTPUT_DELAY_1H_IDX 	 3
#define DDR3_DQ_OUTPUT_DELAY_2L_IDX 	 4
#define DDR3_DQ_OUTPUT_DELAY_2H_IDX 	 5
#define DDR3_DQ_OUTPUT_DELAY_3L_IDX 	 6
#define DDR3_DQ_OUTPUT_DELAY_3H_IDX 	 7
#define DDR3_DQS_OUTPUT_DELAY_IDX	     8
#define DDR3_DQM_DELAY_IDX			     9


#define DRAMTAG_0 0x2F4B544D
#define DRAMTAG_1 0x2F565444
#define DRAMTAG_2 0x4D415244
#define DRAMTAG_3 0x3031562F

#define DRAM_PARAMS_ADDR    (0x3D00000)    //61MB
#define LOADER_DRAM_ADDR    (0x3E00000)    //62MB
#define DRAM_PARAMS_OFFSET  (0x9800)
#define DRAM_PARAMS_ADDR_AT_SRAM  (0xFB009800)

typedef unsigned char   U8;
typedef unsigned short  U16;
typedef unsigned long   U32;
typedef unsigned int    UINT;
typedef char            S8;
typedef short           S16;
typedef long            S32;
typedef int             SINT;

typedef U32             DWORD;  //32 bits

typedef U8              Data8;
typedef U16             Data16;
typedef U32             Data32;

typedef U8              *PU8;
typedef U16             *PU16;
typedef U32             *PU32;
typedef S8              *PS8;
typedef S16             *PS16;

typedef enum
{
    DRAM_OK = 0, // OK
    DRAM_FAIL    // FAIL
} DRAM_STATUS_T; // DRAM status type

typedef enum
{
    TE_OP_WRITE_READ_CHECK = 0,
    TE_OP_READ_CHECK
} DRAM_TE_OP_T;

typedef struct _RXDQS_PERBIT_DLY_T
{
    S8 first_dqsdly_pass;
    S8 last_dqsdly_pass;
	S8 dqsdly_ok_center;
    U8 best_dqdly;
} RXDQS_PERBIT_DLY_T;

typedef struct _TXDQS_PERBIT_DLY_T
{
    S8 first_dqsdly_l_pass;
    S8 last_dqsdly_l_pass;
    S8 first_dqsdly_r_pass;
    S8 last_dqsdly_r_pass;
    S8 dqsdly_ok_center;
} TXDQS_PERBIT_DLY_T;

// timeout for TE2: (CMP_CPT_POLLING_PERIOD X MAX_CMP_CPT_WAIT_LOOP) 
// complete flag, need to double check with DE
#define CMP_CPT_POLLING_PERIOD 10
#define MAX_CMP_CPT_WAIT_LOOP 10000   // max loop

/************************ Bit Process *************************/
#define mcBITL(b)               (1L << (b))
#define mcBIT(b)                (1L << (b))
#define mcMASK(w)               (mcBIT(w) - 1)
#define mcMASKS(w, b)           (mcMASK(w) << (b))
#define mcFIELD(val, msk, pos)  (((val) << (pos)) & (msk))

#define mcSET_MASK(a, b)        ((a) |= (b))
#define mcCLR_MASK(a, b)        ((a) &= (~(b)))
#define mcCHK_MASK(a, b)        ((a) & (b))
//#define mcSET_BIT(a, b)         mcSET_MASK(a, mcBIT(b))
#define mcSET_BIT(a, b)         ((a) |= ((UINT32)1L<<(b)))
//#define mcCLR_BIT(a, b)         mcCLR_MASK(a, mcBIT(b))
#define mcCLR_BIT(a, b)         ((a) &= (~((UINT32)1L<<(b))))
#define mcCHK_BIT1(a, b)        ((a) & mcBIT(b))
#define mcCHK_BITM(a, b, m)     (((a) >> (b)) & (m))
#define mcCHK_BITS(a, b, w)     mcCHK_BITM(a, b, mcMASK(w))
//#define mcTEST_BIT(a, b)        mcCHK_BITM(a, b, 1)
#define mcTEST_BIT(a, b)        mcCHK_BIT1(a, b)
#define mcCHG_BIT(a, b)         ((a) ^= mcBIT(b))

#define mcSET_FIELD0(var, val, msk, pos)    mcSET_MASK(var, mcFIELD(val, msk, pos))

#define mcSET_FIELD(var, value, mask, pos)  \
{                                           \
    mcCLR_MASK(var, mask);                  \
    mcSET_FIELD0(var, value, mask, pos);    \
}

#define mcGET_FIELD(var, mask, pos)     (((var) & (mask)) >> (pos))


/*----------------------------------------------------------------------------*/
/*! @struct DRAM_CFG_T
 *  @brief
 *
 *  @code
 * typedef struct _DRAM_CFG_T
 * {
 *   UINT8       ui1_ssc_range;
 *   UINT8       ui1_ssc_modulation;
 *   UINT8       ui1_clk_driving;
 *   UINT8       ui1_clk_delay;
 *   UINT8       ui1_cmd_driving;
 *   UINT8       ui1_cmd_delay;
 *   UINT8       ui1_wdqs_driving;
 *   UINT8       ui1_wdqs_delay;
 *   UINT8       ui1_wdq_driving;
 *   UINT8       ui1_wdq_delay;    
 * } DRAM_CFG_T;
 *  @endcode
 *
 *  @li@c  ui1_ssc_range               - spread spectrum clock: range 0 ~ 100: 0% ~ +-1%. 0 = disable SSC.
 *  @li@c  ui1_ssc_modulation       - spread spectrum modulation: range 0 ~ 60: 0Khz ~ 60Khz. 0 = disable SSC.
 *  @li@c  ui1_clk_driving               - clock driving: range 0 ~ 15.
 *  @li@c  ui1_clk_delay                 - clock delay: range 0 ~ 15.
 *  @li@c  ui1_cmd_driving             - command driving: range 0 ~ 15.
 *  @li@c  ui1_cmd_delay               - command delay: range 0 ~ 15.
 *  @li@c  ui1_wdqs_driving            - write DQS driving: range 0 ~ 15.
 *  @li@c  ui1_wdqs0_delay             - write DQS0 delay: range 0 ~ 15.
 *  @li@c  ui1_wdqs1_delay             - write DQS1 delay: range 0 ~ 15.
 *  @li@c  ui1_wdqs2_delay             - write DQS2 delay: range 0 ~ 15.
 *  @li@c  ui1_wdqs3_delay             - write DQS3 delay: range 0 ~ 15. 
 *  @li@c  ui1_wdq_driving              - write DQ driving: range 0 ~ 15.
 *  @li@c  ui1_wdq0_delay                - write DQ bit  0 ~  7 delay: range 0 ~ 15.
 *  @li@c  ui1_wdq1_delay                - write DQ bit  8 ~ 15 delay: range 0 ~ 15.
 *  @li@c  ui1_wdq2_delay                - write DQ bit 16 ~ 23 delay: range 0 ~ 15.
 *  @li@c  ui1_wdq3_delay                - write DQ bit 24 ~ 31 delay: range 0 ~ 15.
 */
/*----------------------------------------------------------------------------*/
#if !defined(CC_MTK_PRELOADER) && !defined(CC_MTK_LOADER)
typedef struct _DRAM_CFG_T
{
    UINT8       ui1_ssc_range;
    UINT8       ui1_ssc_modulation;
    UINT8       ui1_clk_driving;
    UINT8       ui1_clk_delay;
    UINT8       ui1_cmd_driving;
    UINT8       ui1_cmd_delay;
    UINT8       ui1_wdqs_driving;
    UINT8       ui1_wdqs0_delay;
    UINT8       ui1_wdqs1_delay;
    UINT8       ui1_wdqs2_delay;    
    UINT8       ui1_wdqs3_delay;        
    UINT8       ui1_wdq_driving;
    UINT8       ui1_wdq0_delay;    
    UINT8       ui1_wdq1_delay;    
    UINT8       ui1_wdq2_delay; 
    UINT8       ui1_wdq3_delay;        
} DRAM_CFG_T;
#endif /* !defined(CC_MTK_PRELOADER) && !defined(CC_MTK_LOADER) */

// DDR related functions.
extern CHAR * DDR_DramTypeString(UINT32 u4Type);
extern UINT32 DDR_IsDqsFail(UINT32 u4DQSth, UINT32 u4DQSVal, UINT32 u4DQSType, UINT32 u4ChipNum);
extern UINT32 DDR_CalibrateDqs(void);
extern UINT32 DDR_CalibrateOutDqs(void);
extern void DDR_SetDramController(void);
extern void DDR_SetAgentPriority(const UINT8 *pu1Prio);
extern void DDR_SetBurstLen(UINT32 u4TimeSlot1, UINT32 u4TimeSlot2);
extern void DDR_SetArbiterTime(UINT8 u1Group, UINT8 u1Time);

/*
    DRAM configuration API for factory mode.
        res_mngr\drv\u_drv_cust.h
        mw_if\drv_cust_api.c
*/ 
#if !defined(CC_MTK_PRELOADER) && !defined(CC_MTK_LOADER)
extern BOOL DDR_SetCustCfg(DRAM_CFG_T *prDdrCfg);
extern BOOL DDR_GetCustCfg(DRAM_CFG_T *prDdrCfg);
#endif /* !defined(CC_MTK_PRELOADER) && !defined(CC_MTK_LOADER) */

// DRAM parameters
UINT32 DRAMGetParameter(UINT8 u1Bank, UINT8 u1Idx);
BOOL DRAMSetParameter(UINT8 u1Bank, UINT8 u1Idx, UINT32 u4Value);
BOOL DRAM_StoreParams(UINT32 u4Addr);

#endif /* X_DRAM_H */
